==> 1.0.3.tar.gz: lib/src/utils/x_date_util.dart <==
/// 一些常用格式参照。可以自定义格式，例如："yyyy/MM/dd HH:mm:ss"，"yyyy/M/d HH:mm:ss"。
/// 格式要求
/// year -> yyyy/yy   month -> MM/M    day -> dd/d
/// hour -> HH/H      minute -> mm/m   second -> ss/s
///
class DateFormats {
  static const String full = "yyyy-MM-dd HH:mm:ss";

  // ignore: non_constant_identifier_names
  static const String y_mo_d_h_m = "yyyy-MM-dd HH:mm";

  // ignore: non_constant_identifier_names
  static const String y_mo_d = "yyyy-MM-dd";

  // ignore: non_constant_identifier_names
  static const String y_mo = "yyyy-MM";

  // ignore: non_constant_identifier_names
  static const String mo_d = "MM-dd";

  // ignore: non_constant_identifier_names
  static const String mo_d_h_m = "MM-dd HH:mm";

  // ignore: non_constant_identifier_names
  static const String h_m_s = "HH:mm:ss";

  // ignore: non_constant_identifier_names
  static const String h_m = "HH:mm";

  // ignore: non_constant_identifier_names
  static const String zh_full = "yyyy年MM月dd日 HH时mm分ss秒";

  // ignore: non_constant_identifier_names
  static const String zh_y_mo_d_h_m = "yyyy年MM月dd日 HH时mm分";

  // ignore: non_constant_identifier_names
  static const String zh_y_mo_d = "yyyy年MM月dd日";

  // ignore: non_constant_identifier_names
  static const String zh_y_mo = "yyyy年MM月";

  // ignore: non_constant_identifier_names
  static const String zh_mo_d = "MM月dd日";

  // ignore: non_constant_identifier_names
  static const String zh_mo_d_h_m = "MM月dd日 HH时mm分";

  // ignore: non_constant_identifier_names
  static const String zh_h_m_s = "HH时mm分ss秒";

  // ignore: non_constant_identifier_names
  static const String zh_h_m = "HH时mm分";
}

/// Date Util.
class XDateUtil {
  /// get DateTime By DateStr.
  static DateTime? getDateTime(String dateStr, {bool? isUtc}) {
    DateTime? dateTime = DateTime.tryParse(dateStr);
    if (isUtc != null) {
      if (isUtc) {
        dateTime = dateTime?.toUtc();
      } else {
        dateTime = dateTime?.toLocal();
      }
    }
    return dateTime;
  }

  /// get DateTime By Milliseconds.
  static DateTime getDateTimeByMs(int ms, {bool isUtc = false}) {
    return DateTime.fromMillisecondsSinceEpoch(ms, isUtc: isUtc);
  }

  /// get DateMilliseconds By DateStr.
  static int? getDateMsByTimeStr(String dateStr) {
    DateTime? dateTime = DateTime.tryParse(dateStr);
    return dateTime?.millisecondsSinceEpoch;
  }

  /// get Now Date Milliseconds.
  static int getNowDateMs() {
    return DateTime
        .now()
        .millisecondsSinceEpoch;
  }

  /// get Now Date Str.(yyyy-MM-dd HH:mm:ss)
  static String getNowDateStr({String format = DateFormats.full}) {
    return formatDate(DateTime.now(), format: format);
  }

  /// format date by milliseconds.
  /// milliseconds 日期毫秒
  static String formatDateMs(int ms, {bool isUtc = false, String format = DateFormats.full}) {
    return formatDate(getDateTimeByMs(ms, isUtc: isUtc), format: format);
  }

  /// format date by date str.
  /// dateStr 日期字符串
  static String? formatDateStr(String dateStr, {bool? isUtc, String format = DateFormats.full}) {
    final dateTime = getDateTime(dateStr, isUtc: isUtc);
    if (dateTime != null) {
      return formatDate(dateTime, format: format);
    } else {
      return null;
    }
  }

  /// format date by DateTime.
  /// format 转换格式(已提供常用格式 DateFormats，可以自定义格式："yyyy/MM/dd HH:mm:ss")
  /// 格式要求
  /// year -> yyyy/yy   month -> MM/M    day -> dd/d
  /// hour -> HH/H      minute -> mm/m   second -> ss/s
  static String formatDate(DateTime dateTime, {String format = DateFormats.full}) {
    if (format.contains("yy")) {
      String year = dateTime.year.toString();
      if (format.contains("yyyy")) {
        format = format.replaceAll("yyyy", year);
      } else {
        format = format.replaceAll("yy", year.substring(year.length - 2, year.length));
      }
    }

    format = _comFormat(dateTime.month, format, 'M', 'MM');
    format = _comFormat(dateTime.day, format, 'd', 'dd');
    format = _comFormat(dateTime.hour, format, 'H', 'HH');
    format = _comFormat(dateTime.minute, format, 'm', 'mm');
    format = _comFormat(dateTime.second, format, 's', 'ss');
    format = _comFormat(dateTime.millisecond, format, 'S', 'SSS');

    return format;
  }

  /// 显示 12小时制时间
  static String formatTime12(DateTime dateTime, {String format = 'HH:mm'}) {
    if (dateTime.hour == 0) {
      format = _comFormatHour('半夜', 12, format);
    } else if (dateTime.hour < 6) {
      format = _comFormatHour('凌晨', dateTime.hour, format);
    } else if (dateTime.hour < 12) {
      format = _comFormatHour('上午', dateTime.hour, format);
    } else if (dateTime.hour == 12) {
      format = _comFormatHour('中午', 12, format);
    } else if (dateTime.hour < 18) {
      format = _comFormatHour('下午', dateTime.hour - 12, format);
    } else if (dateTime.hour == 18) {
      format = _comFormatHour('傍晚', 6, format);
    } else {
      format = _comFormatHour('晚上', dateTime.hour - 12, format);
    }

    format = _comFormat(dateTime.minute, format, 'm', 'mm');
    return format;
  }

  /// 格式化时间
  static String _comFormatHour(String tip, int value, String format) {
    format = _comFormat(value, format, 'H', 'HH');
    return tip + format;
  }

  /// com format.
  static String _comFormat(int value, String format, String single, String full) {
    if (format.contains(single)) {
      if (format.contains(full)) {
        format = format.replaceAll(full, value < 10 ? '0$value' : value.toString());
      } else {
        format = format.replaceAll(single, value.toString());
      }
    }
    return format;
  }

  /// get WeekDay.
  /// dateTime
  /// isUtc
  /// languageCode zh or en
  /// short
  static String getWeekday(DateTime dateTime, {String languageCode = 'en', bool short = false}) {
    late String weekday;
    switch (dateTime.weekday) {
      case 1:
        weekday = languageCode == 'zh' ? '星期一' : "Monday";
        break;
      case 2:
        weekday = languageCode == 'zh' ? '星期二' : "Tuesday";
        break;
      case 3:
        weekday = languageCode == 'zh' ? '星期三' : "Wednesday";
        break;
      case 4:
        weekday = languageCode == 'zh' ? '星期四' : "Thursday";
        break;
      case 5:
        weekday = languageCode == 'zh' ? '星期五' : "Friday";
        break;
      case 6:
        weekday = languageCode == 'zh' ? '星期六' : "Saturday";
        break;
      case 7:
        weekday = languageCode == 'zh' ? '星期日' : "Sunday";
        break;
    }
    return languageCode == 'zh'
        ? (short ? weekday.replaceAll('星期', '周') : weekday)
        : weekday.substring(0, short ? 3 : weekday.length);
  }

  /// get WeekDay By Milliseconds.
  static String getWeekdayByMs(int milliseconds,
      {bool isUtc = false, String languageCode = 'en', bool short = false}) {
    DateTime dateTime = getDateTimeByMs(milliseconds, isUtc: isUtc);
    return getWeekday(dateTime, languageCode: languageCode, short: short);
  }

  /// month->days.
  static const Map<int, int> MONTH_DAY = {
    1: 31,
    2: 28,
    3: 31,
    4: 30,
    5: 31,
    6: 30,
    7: 31,
    8: 31,
    9: 30,
    10: 31,
    11: 30,
    12: 31,
  };

  /// get day of year.
  /// 在今年的第几天.
  static int getDayOfYear(DateTime dateTime) {
    int year = dateTime.year;
    int month = dateTime.month;
    int days = dateTime.day;
    for (int i = 1; i < month; i++) {
      days = days + MONTH_DAY[i]!;
    }
    if (isLeapYearByYear(year) && month > 2) {
      days = days + 1;
    }
    return days;
  }

  /// 在今年的第几天.
  static int getDayOfYearByMs(int ms, {bool isUtc = false}) {
    return getDayOfYear(DateTime.fromMillisecondsSinceEpoch(ms, isUtc: isUtc));
  }

  /// 是否是当天.
  static bool isToday(int milliseconds, {bool isUtc = false, int? locMs}) {
    if (milliseconds == 0) return false;
    DateTime old = DateTime.fromMillisecondsSinceEpoch(milliseconds, isUtc: isUtc);
    DateTime now;
    if (locMs != null) {
      now = XDateUtil.getDateTimeByMs(locMs);
    } else {
      now = isUtc ? DateTime.now().toUtc() : DateTime.now().toLocal();
    }
    return old.year == now.year && old.month == now.month && old.day == now.day;
  }

  /// 是否是昨天.
  static bool isYesterday(DateTime dateTime, DateTime locDateTime) {
    if (yearIsEqual(dateTime, locDateTime)) {
      int spDay = getDayOfYear(locDateTime) - getDayOfYear(dateTime);
      return spDay == 1;
    } else {
      return ((locDateTime.year - dateTime.year == 1) &&
          dateTime.month == 12 &&
          locDateTime.month == 1 &&
          dateTime.day == 31 &&
          locDateTime.day == 1);
    }
  }

  /// 是否是昨天.
  static bool isYesterdayByMs(int ms, int locMs) {
    return isYesterday(
        DateTime.fromMillisecondsSinceEpoch(ms), DateTime.fromMillisecondsSinceEpoch(locMs));
  }

  /// is Week.
  /// 是否是本周.
  static bool isWeek(int ms, {bool isUtc = false, int? locMs}) {
    if (ms <= 0) {
      return false;
    }
    DateTime _old = DateTime.fromMillisecondsSinceEpoch(ms, isUtc: isUtc);
    DateTime _now;
    if (locMs != null) {
      _now = XDateUtil.getDateTimeByMs(locMs, isUtc: isUtc);
    } else {
      _now = isUtc ? DateTime.now().toUtc() : DateTime.now().toLocal();
    }

    DateTime old = _now.millisecondsSinceEpoch > _old.millisecondsSinceEpoch ? _old : _now;
    DateTime now = _now.millisecondsSinceEpoch > _old.millisecondsSinceEpoch ? _now : _old;
    return (now.weekday >= old.weekday) &&
        (now.millisecondsSinceEpoch - old.millisecondsSinceEpoch <= 7 * 24 * 60 * 60 * 1000);
  }

  /// year is equal.
  /// 是否同年.
  static bool yearIsEqual(DateTime dateTime, DateTime locDateTime) {
    return dateTime.year == locDateTime.year;
  }

  /// year is equal.
  /// 是否同年.
  static bool yearIsEqualByMs(int ms, int locMs) {
    return yearIsEqual(
        DateTime.fromMillisecondsSinceEpoch(ms), DateTime.fromMillisecondsSinceEpoch(locMs));
  }

  /// Return whether it is leap year.
  /// 是否是闰年
  static bool isLeapYear(DateTime dateTime) {
    return isLeapYearByYear(dateTime.year);
  }

  /// Return whether it is leap year.
  /// 是否是闰年
  static bool isLeapYearByYear(int year) {
    return year % 4 == 0 && year % 100 != 0 || year % 400 == 0;
  }

  /// int 转 显示时间
  static String secToTime(int mseconds) {
    if (mseconds == 0) return '00 : 00 : 00';

    int seconds = mseconds ~/ 1000;
    int hour = seconds ~/ 3600;
    int minute = (seconds - hour * 3600) ~/ 60;
    int second = (seconds - hour * 3600 - minute * 60);

    final showHour = hour < 10 ? '0$hour' : hour;
    final showMinute = minute < 10 ? '0$minute' : minute;
    final showSecond = second < 10 ? '0$second' : second;
    return '$showHour : $showMinute : $showSecond';
  }
}

==> 1.0.6.tar.gz: lib/src/utils/x_date_util.dart <==
/// 一些常用格式参照。可以自定义格式，例如："yyyy/MM/dd HH:mm:ss"，"yyyy/M/d HH:mm:ss"。
/// 格式要求
/// year -> yyyy/yy   month -> MM/M    day -> dd/d
/// hour -> HH/H      minute -> mm/m   second -> ss/s
///
// ignore_for_file: constant_identifier_names

class DateFormats {
  static const String full = "yyyy-MM-dd HH:mm:ss";

  // ignore: non_constant_identifier_names
  static const String y_mo_d_h_m = "yyyy-MM-dd HH:mm";

  // ignore: non_constant_identifier_names
  static const String y_mo_d = "yyyy-MM-dd";

  // ignore: non_constant_identifier_names
  static const String y_mo = "yyyy-MM";

  // ignore: non_constant_identifier_names
  static const String mo_d = "MM-dd";

  // ignore: non_constant_identifier_names
  static const String mo_d_h_m = "MM-dd HH:mm";

  // ignore: non_constant_identifier_names
  static const String h_m_s = "HH:mm:ss";

  // ignore: non_constant_identifier_names
  static const String h_m = "HH:mm";

  // ignore: non_constant_identifier_names
  static const String zh_full = "yyyy年MM月dd日 HH时mm分ss秒";

  // ignore: non_constant_identifier_names
  static const String zh_y_mo_d_h_m = "yyyy年MM月dd日 HH时mm分";

  // ignore: non_constant_identifier_names
  static const String zh_y_mo_d = "yyyy年MM月dd日";

  // ignore: non_constant_identifier_names
  static const String zh_y_mo = "yyyy年MM月";

  // ignore: non_constant_identifier_names
  static const String zh_mo_d = "MM月dd日";

  // ignore: non_constant_identifier_names
  static const String zh_mo_d_h_m = "MM月dd日 HH时mm分";

  // ignore: non_constant_identifier_names
  static const String zh_h_m_s = "HH时mm分ss秒";

  // ignore: non_constant_identifier_names
  static const String zh_h_m = "HH时mm分";
}

/// Date Util.
class XDateUtil {
  /// 通过字符串获取 DateTime
  static DateTime? getDateTime(String dateStr, {bool? isUtc}) {
    DateTime? dateTime = DateTime.tryParse(dateStr);
    if (isUtc != null) {
      if (isUtc) {
        dateTime = dateTime?.toUtc();
      } else {
        dateTime = dateTime?.toLocal();
      }
    }
    return dateTime;
  }

  /// 通过 毫秒 获取 DateTime
  static DateTime getDateTimeByMs(int ms, {bool isUtc = false}) {
    return DateTime.fromMillisecondsSinceEpoch(ms, isUtc: isUtc);
  }

  /// 通过字符串获取 毫秒
  static int? getDateMsByTimeStr(String dateStr) {
    DateTime? dateTime = DateTime.tryParse(dateStr);
    return dateTime?.millisecondsSinceEpoch;
  }

  /// 获取当前时间戳
  static int getNowDateMs() {
    return DateTime.now().millisecondsSinceEpoch;
  }

  /// 获取当前时间字符串.(yyyy-MM-dd HH:mm:ss)
  static String getNowDateStr({String format = DateFormats.full}) {
    return formatDate(DateTime.now(), format: format);
  }

  /// 格式化 时间戳 显示日期
  /// milliseconds 日期毫秒
  static String formatDateMs(int ms,
      {bool isUtc = false, String format = DateFormats.full}) {
    return formatDate(getDateTimeByMs(ms, isUtc: isUtc), format: format);
  }

  /// 格式化 日期 by 字符串
  /// dateStr 日期字符串
  static String? formatDateStr(String dateStr,
      {bool? isUtc, String format = DateFormats.full}) {
    final dateTime = getDateTime(dateStr, isUtc: isUtc);
    if (dateTime != null) {
      return formatDate(dateTime, format: format);
    } else {
      return null;
    }
  }

  /// 格式化日期 DateTime.
  /// format 转换格式(已提供常用格式 DateFormats，可以自定义格式："yyyy/MM/dd HH:mm:ss")
  /// 格式要求
  /// year -> yyyy/yy   month -> MM/M    day -> dd/d
  /// hour -> HH/H      minute -> mm/m   second -> ss/s
  static String formatDate(DateTime dateTime,
      {String format = DateFormats.full}) {
    if (format.contains("yy")) {
      String year = dateTime.year.toString();
      if (format.contains("yyyy")) {
        format = format.replaceAll("yyyy", year);
      } else {
        format = format.replaceAll(
            "yy", year.substring(year.length - 2, year.length));
      }
    }

    format = _comFormat(dateTime.month, format, 'M', 'MM');
    format = _comFormat(dateTime.day, format, 'd', 'dd');
    format = _comFormat(dateTime.hour, format, 'H', 'HH');
    format = _comFormat(dateTime.minute, format, 'm', 'mm');
    format = _comFormat(dateTime.second, format, 's', 'ss');
    format = _comFormat(dateTime.millisecond, format, 'S', 'SSS');

    return format;
  }

  /// 显示 12小时制时间
  static String formatTime12(DateTime dateTime, {String format = 'HH:mm'}) {
    if (dateTime.hour == 0) {
      format = _comFormatHour('半夜', 12, format);
    } else if (dateTime.hour < 6) {
      format = _comFormatHour('凌晨', dateTime.hour, format);
    } else if (dateTime.hour < 12) {
      format = _comFormatHour('上午', dateTime.hour, format);
    } else if (dateTime.hour == 12) {
      format = _comFormatHour('中午', 12, format);
    } else if (dateTime.hour < 18) {
      format = _comFormatHour('下午', dateTime.hour - 12, format);
    } else if (dateTime.hour == 18) {
      format = _comFormatHour('傍晚', 6, format);
    } else {
      format = _comFormatHour('晚上', dateTime.hour - 12, format);
    }

    format = _comFormat(dateTime.minute, format, 'm', 'mm');
    return format;
  }

  /// 格式化时间
  static String _comFormatHour(String tip, int value, String format) {
    format = _comFormat(value, format, 'H', 'HH');
    return tip + format;
  }

  /// com format.
  static String _comFormat(
      int value, String format, String single, String full) {
    if (format.contains(single)) {
      if (format.contains(full)) {
        format =
            format.replaceAll(full, value < 10 ? '0$value' : value.toString());
      } else {
        format = format.replaceAll(single, value.toString());
      }
    }
    return format;
  }

  /// 获取日期
  static String getWeekday(DateTime dateTime,
      {String languageCode = 'zh', bool short = false}) {
    late String weekday;
    switch (dateTime.weekday) {
      case 1:
        weekday = languageCode == 'zh' ? '星期一' : "Monday";
        break;
      case 2:
        weekday = languageCode == 'zh' ? '星期二' : "Tuesday";
        break;
      case 3:
        weekday = languageCode == 'zh' ? '星期三' : "Wednesday";
        break;
      case 4:
        weekday = languageCode == 'zh' ? '星期四' : "Thursday";
        break;
      case 5:
        weekday = languageCode == 'zh' ? '星期五' : "Friday";
        break;
      case 6:
        weekday = languageCode == 'zh' ? '星期六' : "Saturday";
        break;
      case 7:
        weekday = languageCode == 'zh' ? '星期日' : "Sunday";
        break;
    }
    return languageCode == 'zh'
        ? (short ? weekday.replaceAll('星期', '周') : weekday)
        : weekday.substring(0, short ? 3 : weekday.length);
  }

  /// get WeekDay By Milliseconds.
  static String getWeekdayByMs(int milliseconds,
      {bool isUtc = false, String languageCode = 'zh', bool short = false}) {
    DateTime dateTime = getDateTimeByMs(milliseconds, isUtc: isUtc);
    return getWeekday(dateTime, languageCode: languageCode, short: short);
  }

  /// month->days.
  static const Map<int, int> MONTH_DAY = {
    1: 31,
    2: 28,
    3: 31,
    4: 30,
    5: 31,
    6: 30,
    7: 31,
    8: 31,
    9: 30,
    10: 31,
    11: 30,
    12: 31,
  };

  /// 在今年的第几天.
  static int getDayOfYear(DateTime dateTime) {
    int year = dateTime.year;
    int month = dateTime.month;
    int days = dateTime.day;
    for (int i = 1; i < month; i++) {
      days = days + MONTH_DAY[i]!;
    }
    if (isLeapYearByYear(year) && month > 2) {
      days = days + 1;
    }
    return days;
  }

  /// 在今年的第几天.
  static int getDayOfYearByMs(int ms, {bool isUtc = false}) {
    return getDayOfYear(DateTime.fromMillisecondsSinceEpoch(ms, isUtc: isUtc));
  }

  /// 是否是当天.
  static bool isToday(int milliseconds, {bool isUtc = false, int? locMs}) {
    if (milliseconds == 0) return false;
    DateTime old =
        DateTime.fromMillisecondsSinceEpoch(milliseconds, isUtc: isUtc);
    DateTime now;
    if (locMs != null) {
      now = XDateUtil.getDateTimeByMs(locMs);
    } else {
      now = isUtc ? DateTime.now().toUtc() : DateTime.now().toLocal();
    }
    return old.year == now.year && old.month == now.month && old.day == now.day;
  }

  /// 是否是昨天.
  static bool isYesterday(DateTime dateTime, DateTime locDateTime) {
    if (yearIsEqual(dateTime, locDateTime)) {
      int spDay = getDayOfYear(locDateTime) - getDayOfYear(dateTime);
      return spDay == 1;
    } else {
      return ((locDateTime.year - dateTime.year == 1) &&
          dateTime.month == 12 &&
          locDateTime.month == 1 &&
          dateTime.day == 31 &&
          locDateTime.day == 1);
    }
  }

  /// 是否是昨天.
  static bool isYesterdayByMs(int ms, int locMs) {
    return isYesterday(DateTime.fromMillisecondsSinceEpoch(ms),
        DateTime.fromMillisecondsSinceEpoch(locMs));
  }

  /// 是否是本周.
  static bool isWeek(int ms, {bool isUtc = false, int? locMs}) {
    if (ms <= 0) {
      return false;
    }
    DateTime _old = DateTime.fromMillisecondsSinceEpoch(ms, isUtc: isUtc);
    DateTime _now;
    if (locMs != null) {
      _now = XDateUtil.getDateTimeByMs(locMs, isUtc: isUtc);
    } else {
      _now = isUtc ? DateTime.now().toUtc() : DateTime.now().toLocal();
    }

    DateTime old =
        _now.millisecondsSinceEpoch > _old.millisecondsSinceEpoch ? _old : _now;
    DateTime now =
        _now.millisecondsSinceEpoch > _old.millisecondsSinceEpoch ? _now : _old;
    return (now.weekday >= old.weekday) &&
        (now.millisecondsSinceEpoch - old.millisecondsSinceEpoch <=
            7 * 24 * 60 * 60 * 1000);
  }

  /// 是否同年.
  static bool yearIsEqual(DateTime dateTime, DateTime locDateTime) {
    return dateTime.year == locDateTime.year;
  }

  /// 是否同年.
  static bool yearIsEqualByMs(int ms, int locMs) {
    return yearIsEqual(DateTime.fromMillisecondsSinceEpoch(ms),
        DateTime.fromMillisecondsSinceEpoch(locMs));
  }

  /// 是否是闰年
  static bool isLeapYear(DateTime dateTime) {
    return isLeapYearByYear(dateTime.year);
  }

  /// 是否是闰年
  static bool isLeapYearByYear(int year) {
    return year % 4 == 0 && year % 100 != 0 || year % 400 == 0;
  }

  /// int 转 显示时间
  static String secToTime(int mseconds) {
    if (mseconds == 0) return '00 : 00 : 00';

    int seconds = mseconds ~/ 1000;
    int hour = seconds ~/ 3600;
    int minute = (seconds - hour * 3600) ~/ 60;
    int second = (seconds - hour * 3600 - minute * 60);

    final showHour = hour < 10 ? '0$hour' : hour;
    final showMinute = minute < 10 ? '0$minute' : minute;
    final showSecond = second < 10 ? '0$second' : second;
    return '$showHour : $showMinute : $showSecond';
  }
}
